	
#version 330

in vec2 fragmentUV;
in vec3 P;
in vec3 N;

out vec4 fragColor;

uniform sampler2D colorTexture;

uniform samplerCube cm_z_tex;
uniform vec2 near_far;

const int lightCount = 1;
uniform vec3 lightPositions[lightCount];
uniform vec3 cameraPosition;
uniform vec3 objectColor;



uniform vec3 _SpecColor = vec3(1,1,1);
uniform float _Shininess = 200;
uniform vec3 _LightColor0 = vec3(1.0,1.0,1.0);


uniform float transparency;


void main() {
	float smallest_dist_to_light1;
	float smallest_dist_to_light2;
	float smallest_dist_to_light3;
	float smallest_dist_to_light4;
	float smallest_dist_to_light5;
	float smallest_dist_to_light6;
	float smallest_dist_to_light7;
	float smallest_dist_to_light8;
	vec3 l1;
	float curr_fragment_dist_to_light;

	vec3 textureColor = texture(colorTexture, fragmentUV).rgb;
	vec3 normalDirection = normalize(N);

	vec3 viewDirection = normalize(cameraPosition - vec3(P));
    vec3 lightDirection;
    float attenuation;

	vec3 ambientLighting =  vec3(0.9f,0.9f,0.9f) * vec3(objectColor);

	  // initialize total lighting with ambient lighting
    vec3 totalLighting = ambientLighting;

	for (int index = 0; index < lightCount; index++) { // for all light sources

		vec3 vertexToLightSource = vec3(lightPositions[index] - P);
		float distance = length(vertexToLightSource);
		attenuation = 1.0 / distance; // linear attenuation 
		lightDirection = normalize(vertexToLightSource);
	
		vec3 diffuseReflection = attenuation * vec3(_LightColor0) * vec3(objectColor) 
				   * max(0.0, dot(normalDirection, lightDirection));

		 vec3 specularReflection;
				if (dot(normalDirection, lightDirection) < 0.0) 
				   // light source on the wrong side?
				{
				   specularReflection = vec3(0.0, 0.0, 0.0); 
					  // no specular reflection
				}
				else // light source on the right side
				{
				   specularReflection = attenuation * vec3(_LightColor0) 
					  * vec3(_SpecColor) * pow(max(0.0, dot(
					  reflect(-lightDirection, normalDirection), 
					  viewDirection)), _Shininess);
				} 

		totalLighting = totalLighting + diffuseReflection + specularReflection;
		// totalIntensity = ambientLighting + diffuseReflection + specularReflection;

		
		vec3 cm_lookup_vec = P.xyz - lightPositions[index].xyz;

		//calculate shadow
		l1 = cm_lookup_vec;
		float m1 = max(max(l1.x, l1.y), l1.z);
		l1 = l1 - vec3(m1, m1, m1);
		l1 = l1 / (l1+vec3(0.001, 0.001, 0.001));
		l1 = l1 * (1.0/300.0);

		//l1 = vec3(1.0/600.0, 1.0/600.0, 0);

		vec3 offset1 = l1;
		vec3 offset2 = -l1;

		vec3 offset3 = l1;
		offset3.x = offset3.x*-1;

		vec3 offset4 = -offset3;

		vec3 offset5 = offset1 + offset3;
		vec3 offset6 = offset1 + offset4;
		vec3 offset7 = offset2 + offset3;
		vec3 offset8 = offset2 + offset4;

		smallest_dist_to_light1 = texture(cm_z_tex, cm_lookup_vec + offset1).r;
		smallest_dist_to_light1 *= near_far.y;
		smallest_dist_to_light2 = texture(cm_z_tex, cm_lookup_vec + offset2).r;
		smallest_dist_to_light2 *= near_far.y;
		smallest_dist_to_light3 = texture(cm_z_tex, cm_lookup_vec + offset3).r;
		smallest_dist_to_light3 *= near_far.y;
		smallest_dist_to_light4 = texture(cm_z_tex, cm_lookup_vec + offset4).r;
		smallest_dist_to_light4 *= near_far.y;
		smallest_dist_to_light5 = texture(cm_z_tex, cm_lookup_vec + offset5).r;
		smallest_dist_to_light5 *= near_far.y;
		smallest_dist_to_light6 = texture(cm_z_tex, cm_lookup_vec + offset6).r;
		smallest_dist_to_light6 *= near_far.y;
		smallest_dist_to_light7 = texture(cm_z_tex, cm_lookup_vec + offset7).r;
		smallest_dist_to_light7 *= near_far.y;
		smallest_dist_to_light8 = texture(cm_z_tex, cm_lookup_vec + offset8).r;
		smallest_dist_to_light8 *= near_far.y;

		curr_fragment_dist_to_light = length(cm_lookup_vec);
	}
	float eps = 0.05;
	//fragColor = vec4(totalIntensity + ambientLighting, 1.0) * vec4(textureColor, 1.0);

	float shadowfac = 1.0;

	if(smallest_dist_to_light1 + eps < curr_fragment_dist_to_light)
		shadowfac = shadowfac - 0.075;
	if(smallest_dist_to_light2 + eps < curr_fragment_dist_to_light)
		shadowfac = shadowfac - 0.075;
	if(smallest_dist_to_light3 + eps < curr_fragment_dist_to_light)
		shadowfac = shadowfac - 0.075;
	if(smallest_dist_to_light4 + eps < curr_fragment_dist_to_light)
		shadowfac = shadowfac - 0.075;
	if(smallest_dist_to_light5 + eps < curr_fragment_dist_to_light)
		shadowfac = shadowfac - 0.075;
	if(smallest_dist_to_light6 + eps < curr_fragment_dist_to_light)
		shadowfac = shadowfac - 0.075;
	if(smallest_dist_to_light7 + eps < curr_fragment_dist_to_light)
		shadowfac = shadowfac - 0.075;
	if(smallest_dist_to_light8 + eps < curr_fragment_dist_to_light)
		shadowfac = shadowfac - 0.075;
		
		
		
		
	fragColor = vec4(totalLighting, 1.0) * vec4(textureColor, 1.0) * shadowfac;
	//fragColor = vec4(l1.x, l1.y, l1.z, 1.0);
	
	
	fragColor.a = transparency;
}
	
	
